@@ -137,6 +137,7 @@ static int corosync_main_config_format_set (
 	char new_format_buffer[PATH_MAX];
 	char *value = NULL;
 	int err = 0;
+	char timestamp_str_to_add[8];
 
 	if (map_get_string("logging.fileline", &value) == CS_OK) {
 		if (strcmp (value, "on") == 0) {
@@ -196,15 +197,16 @@ static int corosync_main_config_format_set (
 		goto parse_error;
 	}
 
+	memset(timestamp_str_to_add, 0, sizeof(timestamp_str_to_add));
+
 	if (map_get_string("logging.timestamp", &value) == CS_OK) {
 		if (strcmp (value, "on") == 0) {
-			if(!insert_into_buffer(new_format_buffer,
-					sizeof(new_format_buffer),
-					"%t ", NULL)) {
-				err = logsys_format_set(new_format_buffer);
-			}
-		} else
-		if (strcmp (value, "off") == 0) {
+			strcpy(timestamp_str_to_add, "%t");
+#ifdef QB_FEATURE_LOG_HIRES_TIMESTAMPS
+		} else if (strcmp (value, "hires") == 0) {
+			strcpy(timestamp_str_to_add, "%T");
+#endif
+		} else if (strcmp (value, "off") == 0) {
 			/* nothing to do here */
 		} else {
 			error_reason = "unknown value for timestamp";
@@ -213,6 +215,23 @@ static int corosync_main_config_format_set (
 		}
 
 		free(value);
+	} else {
+		/*
+		 * Display hires timestamp by default, otherwise standard timestamp
+		 */
+#ifdef QB_FEATURE_LOG_HIRES_TIMESTAMPS
+		strcpy(timestamp_str_to_add, "%T");
+#else
+		strcpy(timestamp_str_to_add, "%t");
+#endif
+	}
+
+	if(strcmp(timestamp_str_to_add, "") != 0) {
+		strcat(timestamp_str_to_add, " ");
+		if (insert_into_buffer(new_format_buffer, sizeof(new_format_buffer),
+		    timestamp_str_to_add, NULL) == 0) {
+			err = logsys_format_set(new_format_buffer);
+		}
 	}
 
 	if (err) {
@@ -523,6 +542,7 @@ static int corosync_main_config_set (
 	else {
 		if (logsys_config_debug_set (subsys, LOGSYS_DEBUG_OFF) < 0) {
 			error_reason = "unable to set debug off";
+			free(value);
 			goto parse_error;
 		}
 	}
@@ -584,7 +604,14 @@ static int corosync_main_config_read_logging (
 			continue ;
 		}
 
-		snprintf(key_item, MAP_KEYNAME_MAXLEN, "logging.logger_subsys.%s", key_subsys);
+		if (snprintf(key_item, MAP_KEYNAME_MAXLEN, "logging.logger_subsys.%s",
+		    key_subsys) >= MAP_KEYNAME_MAXLEN) {
+			/*
+			 * This should never happen
+			 */
+			error_reason = "Can't snprintf logger_subsys key_item";
+			goto parse_error;
+		}
 
 		if (corosync_main_config_set(key_item, key_subsys, &error_reason) < 0) {
 			goto parse_error;
