@@ -74,12 +74,14 @@
 #define FAIL_TO_RECV_CONST			2500
 #define	SEQNO_UNCHANGED_CONST			30
 #define MINIMUM_TIMEOUT				(int)(1000/HZ)*3
+#define MINIMUM_TIMEOUT_HOLD			(int)(MINIMUM_TIMEOUT * 0.8 - (1000/HZ))
 #define MAX_NETWORK_DELAY			50
 #define WINDOW_SIZE				50
 #define MAX_MESSAGES				17
 #define MISS_COUNT_CONST			5
+#define BLOCK_UNLISTED_IPS			1
 
-/* These currently match the defaults in libknet.h */
+/* Currently all but PONG_COUNT match the defaults in libknet.h */
 #define KNET_PING_INTERVAL                      1000
 #define KNET_PING_TIMEOUT                       2000
 #define KNET_PING_PRECISION                     2048
@@ -137,6 +139,8 @@ static void *totem_get_param_by_name(struct totem_config *totem_config, const ch
 		return &totem_config->knet_compression_level;
 	if (strcmp(param_name, "totem.knet_compression_model") == 0)
 		return &totem_config->knet_compression_model;
+	if (strcmp(param_name, "totem.block_unlisted_ips") == 0)
+		return &totem_config->block_unlisted_ips;
 
 	return NULL;
 }
@@ -234,6 +238,55 @@ static void totem_volatile_config_set_string_value (struct totem_config *totem_c
 	icmap_set_string(runtime_key_name, (char *)*config_value);
 }
 
+/*
+ * Read string value stored in key_name from icmap, use it as a boolean (yes/no) type, convert it
+ * to integer value (1/0) and store into totem_config.
+ *
+ * If key is not found or key_name == delete_key default value is used
+ * and stored into totem_config.
+ */
+static void totem_volatile_config_set_boolean_value (struct totem_config *totem_config,
+	const char *key_name, const char *deleted_key, unsigned int default_value)
+{
+	char runtime_key_name[ICMAP_KEYNAME_MAXLEN];
+	char *str;
+	int val;
+
+	str = NULL;
+	val = default_value;
+
+	if ((deleted_key != NULL && strcmp(deleted_key, key_name) == 0) ||
+	    (icmap_get_string(key_name, &str) != CS_OK)) {
+		/*
+		 * Do nothing. str is NULL (icmap_get_string ether not called or
+		 * not changed str).
+		 */
+	} else {
+		if (strcmp(str, "yes") == 0) {
+			val = 1;
+		} else if (strcmp(str, "no") == 0) {
+			val = 0;
+		}
+		free(str);
+	}
+
+	/*
+	 * Store totem_config value to cmap runtime section
+	 */
+	if (strlen("runtime.config.") + strlen(key_name) >= ICMAP_KEYNAME_MAXLEN) {
+		/*
+		 * This shouldn't happen
+		 */
+		return ;
+	}
+
+	strcpy(runtime_key_name, "runtime.config.");
+	strcat(runtime_key_name, key_name);
+
+	*(uint32_t *)totem_get_param_by_name(totem_config, key_name) = val;
+
+	icmap_set_uint32(runtime_key_name, val);
+}
 
 /*
  * Read and validate config values from cmap and store them into totem_config. If key doesn't exists,
@@ -301,7 +354,8 @@ static void totem_volatile_config_read (struct totem_config *totem_config, const
 
 	totem_volatile_config_set_string_value(totem_config, "totem.knet_compression_model", deleted_key, "none");
 
-
+	totem_volatile_config_set_boolean_value(totem_config, "totem.block_unlisted_ips", deleted_key,
+	    BLOCK_UNLISTED_IPS);
 }
 
 static int totem_volatile_config_validate (
@@ -313,6 +367,7 @@ static int totem_volatile_config_validate (
 	char name_key[ICMAP_KEYNAME_MAXLEN];
 	char *name_str;
 	int i, num_configured, members;
+	uint32_t tmp_config_value;
 
 	if (totem_config->max_network_delay < MINIMUM_TIMEOUT) {
 		snprintf (local_error_reason, sizeof(local_error_reason),
@@ -336,16 +391,22 @@ static int totem_volatile_config_validate (
 	}
 
 	if (totem_config->token_retransmit_timeout < MINIMUM_TIMEOUT) {
-		snprintf (local_error_reason, sizeof(local_error_reason),
-			"The token retransmit timeout parameter (%d ms) may not be less than (%d ms).",
-			totem_config->token_retransmit_timeout, MINIMUM_TIMEOUT);
-		goto parse_error;
+		if (icmap_get_uint32("totem.token_retransmit", &tmp_config_value) == CS_OK) {
+			snprintf (local_error_reason, sizeof(local_error_reason),
+				"The token retransmit timeout parameter (%d ms) may not be less than (%d ms).",
+				totem_config->token_retransmit_timeout, MINIMUM_TIMEOUT);
+			goto parse_error;
+		} else {
+			snprintf (local_error_reason, sizeof(local_error_reason),
+				"Not appropriate token or token_retransmits_before_loss_const value set");
+			goto parse_error;
+		}
 	}
 
-	if (totem_config->token_hold_timeout < MINIMUM_TIMEOUT) {
+	if (totem_config->token_hold_timeout < MINIMUM_TIMEOUT_HOLD) {
 		snprintf (local_error_reason, sizeof(local_error_reason),
 			"The token hold timeout parameter (%d ms) may not be less than (%d ms).",
-			totem_config->token_hold_timeout, MINIMUM_TIMEOUT);
+			totem_config->token_hold_timeout, MINIMUM_TIMEOUT_HOLD);
 		goto parse_error;
 	}
 
@@ -386,16 +447,22 @@ static int totem_volatile_config_validate (
 
 	/* Check that we have nodelist 'name' if there is more than one link */
 	num_configured = 0;
+	members = -1;
 	for (i = 0; i < INTERFACE_MAX; i++) {
 		if (totem_config->interfaces[i].configured) {
+			if (num_configured == 0) {
+				members = totem_config->interfaces[i].member_count;
+			}
 			num_configured++;
 		}
 	}
 
 	if (num_configured > 1) {
-		members = totem_config->interfaces[0].member_count;
-
-		for (i=0; i<totem_config->interfaces[0].member_count; i++) {
+		/*
+		 * This assert is here just to make compiler happy
+		 */
+		assert(members != -1);
+		for (i=0; i < members; i++) {
 			snprintf(name_key, sizeof(name_key), "nodelist.node.%d.name", i);
 
 			if (icmap_get_string(name_key, &name_str) != CS_OK) {
@@ -405,7 +472,10 @@ static int totem_volatile_config_validate (
 			}
 		}
 
-		for (i=0; i<num_configured; i++) {
+		for (i=0; i < INTERFACE_MAX; i++) {
+			if (!totem_config->interfaces[i].configured) {
+				continue;
+			}
 			if (totem_config->interfaces[i].member_count != members) {
 				snprintf (local_error_reason, sizeof(local_error_reason),
 					  "Not all nodes have the same number of links");
@@ -450,6 +520,14 @@ static int totem_get_crypto(struct totem_config *totem_config, const char **erro
 		tmp_model = "nss";
 	}
 
+	if (icmap_get_string("totem.secauth", &str) == CS_OK) {
+		if (strcmp(str, "on") == 0) {
+			tmp_cipher = "aes256";
+			tmp_hash = "sha256";
+		}
+		free(str);
+	}
+
 	if (icmap_get_string("totem.crypto_cipher", &str) == CS_OK) {
 		if (strcmp(str, "none") == 0) {
 			tmp_cipher = "none";
@@ -463,9 +541,6 @@ static int totem_get_crypto(struct totem_config *totem_config, const char **erro
 		if (strcmp(str, "aes128") == 0) {
 			tmp_cipher = "aes128";
 		}
-		if (strcmp(str, "3des") == 0) {
-			tmp_cipher = "3des";
-		}
 		free(str);
 	}
 
@@ -555,27 +630,31 @@ static int nodelist_byname(const char *find_name, int strip_domain)
 	return -1;
 }
 
-/* Compare two addresses */
-static int ipaddr_equal(struct sockaddr_storage *addr1, struct sockaddr_storage *addr2)
+/* Compare two addresses - only address part (sin_addr/sin6_addr) is checked */
+static int ipaddr_equal(const struct sockaddr *addr1, const struct sockaddr *addr2)
 {
 	int addrlen = 0;
+	const void *addr1p, *addr2p;
 
-	if (addr1->ss_family != addr2->ss_family)
+	if (addr1->sa_family != addr2->sa_family)
 		return 0;
 
-	if (addr1->ss_family == AF_INET) {
-		addrlen = sizeof(struct sockaddr_in);
+	switch (addr1->sa_family) {
+	case AF_INET:
+		addrlen = sizeof(struct in_addr);
+		addr1p = &((struct sockaddr_in *)addr1)->sin_addr;
+		addr2p = &((struct sockaddr_in *)addr2)->sin_addr;
+		break;
+	case AF_INET6:
+		addrlen = sizeof(struct in6_addr);
+		addr1p = &((struct sockaddr_in6 *)addr1)->sin6_addr;
+		addr2p = &((struct sockaddr_in6 *)addr2)->sin6_addr;
+		break;
+	default:
+		assert(0);
 	}
-	if (addr1->ss_family == AF_INET6) {
-		addrlen = sizeof(struct sockaddr_in6);
-	}
-	assert(addrlen);
-
-	if (memcmp(addr1, addr2, addrlen) == 0)
-		return 1;
-	else
-		return 0;
 
+	return (memcmp(addr1p, addr2p, addrlen) == 0);
 }
 
 
@@ -755,8 +834,7 @@ static int find_local_node(int use_cache)
 		for (rp = result; rp != NULL; rp = rp->ai_next) {
 			for (ifa = ifa_list; ifa; ifa = ifa->ifa_next) {
 				if (ifa->ifa_addr &&
-				    ipaddr_equal((struct sockaddr_storage *)rp->ai_addr,
-						 (struct sockaddr_storage *)ifa->ifa_addr)) {
+				    ipaddr_equal(rp->ai_addr, ifa->ifa_addr)) {
 					freeaddrinfo(result);
 					found = 1;
 					goto out2;
@@ -1147,7 +1225,8 @@ static void check_things_have_not_changed(struct totem_config *totem_config)
 	int changed = 0;
 
 	for (i = 0; i<INTERFACE_MAX; i++) {
-		if (totem_config->interfaces[i].configured) {
+		if (totem_config->interfaces[i].configured &&
+		    totem_config->orig_interfaces[i].configured) {
 			if (totem_config->interfaces[i].knet_transport !=
 			    totem_config->orig_interfaces[i].knet_transport) {
 				log_printf(LOGSYS_LEVEL_ERROR,
@@ -1253,8 +1332,18 @@ static int put_nodelist_members_to_config(struct totem_config *totem_config, int
 				snprintf(tmp_key, ICMAP_KEYNAME_MAXLEN, "nodelist.node.%u.ring0_addr", node_pos);
 				if (icmap_get_string(tmp_key, &str) == CS_OK) {
 					nodeid = generate_nodeid(totem_config, str);
+					if (nodeid == -1) {
+						sprintf(error_string_response,
+						    "An IPV6 network requires that a node ID be specified "
+						    "for address '%s'.", node_addr_str);
+						*error_string = error_string_response;
+						free(str);
+
+						return (-1);
+					}
+
 					log_printf(LOGSYS_LEVEL_DEBUG,
-						   "Generated nodeid = 0x%x for %s", nodeid, str);
+						   "Generated nodeid = " CS_PRI_NODE_ID " for %s", nodeid, str);
 					free(str);
 				}
 			}
@@ -1355,6 +1444,7 @@ static int get_interface_params(struct totem_config *totem_config,
 	uint32_t u32;
 	char *str;
 	char *cluster_name = NULL;
+	enum totem_ip_version_enum tmp_ip_version = TOTEM_IP_VERSION_4;
 
 	if (reload) {
 		for (i=0; i<INTERFACE_MAX; i++) {
@@ -1445,9 +1535,16 @@ static int get_interface_params(struct totem_config *totem_config,
 				 * udpu doesn't need mcastaddr and validity of mcastaddr for udp is
 				 * checked later anyway.
 				 */
+
+				if (totem_config->interfaces[0].bindnet.family == AF_INET) {
+					tmp_ip_version = TOTEM_IP_VERSION_4;
+				} else if (totem_config->interfaces[0].bindnet.family == AF_INET6) {
+					tmp_ip_version = TOTEM_IP_VERSION_6;
+				}
+
 				(void)get_cluster_mcast_addr (cluster_name,
 							      linknumber,
-							      totem_config->ip_version,
+							      tmp_ip_version,
 							      &totem_config->interfaces[linknumber].mcast_addr);
 			}
 
@@ -1745,6 +1842,13 @@ extern int totem_config_read (
 				icmap_get_string(tmp_key, &str);
 
 				totem_config->node_id = generate_nodeid(totem_config, str);
+				if (totem_config->node_id == -1) {
+					*error_string = "An IPV6 network requires that a node ID be specified";
+
+					free(str);
+					return (-1);
+				}
+
 				totem_config->interfaces[0].member_list[local_node_pos].nodeid = totem_config->node_id;
 
 				free(str);
