@@ -59,17 +59,22 @@ extern int totemnet_initialize (
 	void **net_context,
 	struct totem_config *totem_config,
 	totemsrp_stats_t *stats,
-	int interface_no,
 	void *context,
 
 	void (*deliver_fn) (
 		void *context,
 		const void *msg,
-		unsigned int msg_len),
+		unsigned int msg_len,
+		const struct sockaddr_storage *system_from),
 
 	void (*iface_change_fn) (
 		void *context,
-		const struct totem_ip_address *iface_address),
+		const struct totem_ip_address *iface_address,
+		unsigned int iface_no),
+
+	void (*mtu_changed) (
+		void *context,
+		int net_mtu),
 
 	void (*target_set_completed) (
 		void *context));
@@ -101,21 +106,31 @@ extern int totemnet_recv_flush (void *net_context);
 
 extern int totemnet_send_flush (void *net_context);
 
+extern int totemnet_iface_set (void *net_context,
+       const struct totem_ip_address *interface_addr,
+       unsigned short ip_port,
+       unsigned int iface_no);
+
 extern int totemnet_iface_check (void *net_context);
 
 extern int totemnet_finalize (void *net_context);
 
 extern int totemnet_net_mtu_adjust (void *net_context, struct totem_config *totem_config);
 
+extern int totemnet_reconfigure (void *net_context, struct totem_config *totem_config);
+
+extern void totemnet_stats_clear (void *net_context);
+
 extern const char *totemnet_iface_print (void *net_context);
 
-extern int totemnet_iface_get (
+extern int totemnet_ifaces_get (
 	void *net_context,
-	struct totem_ip_address *addr);
+	char ***status,
+	unsigned int *iface_count);
 
 extern int totemnet_token_target_set (
 	void *net_context,
-	const struct totem_ip_address *token_target);
+	unsigned int target_nodeid);
 
 extern int totemnet_crypto_set (
 	void *net_context,
@@ -127,10 +142,18 @@ extern int totemnet_recv_mcast_empty (
 
 extern int totemnet_member_add (
 	void *net_context,
-	const struct totem_ip_address *member);
+	const struct totem_ip_address *local,
+	const struct totem_ip_address *member,
+	int ring_no);
 
 extern int totemnet_member_remove (
 	void *net_context,
-	const struct totem_ip_address *member);
+	const struct totem_ip_address *member,
+	int ring_no);
+
+extern int totemnet_member_set_active (
+	void *net_context,
+	const struct totem_ip_address *member,
+	int active);
 
 #endif /* TOTEMNET_H_DEFINED */
