@@ -53,7 +53,7 @@ int totemsrp_initialize (
 	qb_loop_t *poll_handle,
 	void **srp_context,
 	struct totem_config *totem_config,
-	totemmrp_stats_t *stats,
+	totempg_stats_t *stats,
 
 	void (*deliver_fn) (
 		unsigned int nodeid,
@@ -104,6 +104,7 @@ extern void totemsrp_net_mtu_adjust (struct totem_config *totem_config);
 extern int totemsrp_ifaces_get (
 	void *srp_context,
 	unsigned int nodeid,
+	unsigned int *interface_id,
 	struct totem_ip_address *interfaces,
 	unsigned int interfaces_size,
 	char ***status,
@@ -120,27 +121,40 @@ extern int totemsrp_crypto_set (
 	const char *cipher_type,
 	const char *hash_type);
 
-extern int totemsrp_ring_reenable (
-	void *srp_context);
-
 void totemsrp_service_ready_register (
 	void *srp_context,
 	void (*totem_service_ready) (void));
 
+extern int totemsrp_iface_set (
+	void *srp_context,
+	const struct totem_ip_address *interface_addr,
+	unsigned short ip_port,
+	unsigned int iface_no);
+
 extern int totemsrp_member_add (
 	void *srp_context,
 	const struct totem_ip_address *member,
 	int ring_no);
-	
+
 extern int totemsrp_member_remove (
 	void *srp_context,
 	const struct totem_ip_address *member,
 	int ring_no);
-	
+
 void totemsrp_threaded_mode_enable (
 	void *srp_context);
 
 void totemsrp_trans_ack (
 	void *srp_context);
 
+int totemsrp_reconfigure (
+	void *context,
+	struct totem_config *totem_config);
+
+void totemsrp_stats_clear (
+	void *srp_context, int flags);
+
+void totemsrp_force_gather (
+	void *context);
+
 #endif /* TOTEMSRP_H_DEFINED */
