@@ -73,6 +73,7 @@ enum {
 	CS_NTF_SNMP,
 	CS_NTF_DBUS,
 	CS_NTF_FG,
+	CS_NTF_NODNS,
 	CS_NTF_MAX,
 };
 static int conf[CS_NTF_MAX];
@@ -237,6 +238,7 @@ static void _cs_cmap_members_key_changed (
 
 	if (NULL == key_name) {
 		qb_log(LOG_ERR, "key_name: nil");
+		return ;
 	}
 
 	res = sscanf(key_name, "runtime.members.%u.%s", &nodeid, tmp_key);
@@ -249,7 +251,7 @@ static void _cs_cmap_members_key_changed (
 
 	res = snprintf(tmp_key, CMAP_KEYNAME_MAXLEN, "runtime.members.%u.ip", nodeid);
 	if (res <= 0 || res >= CMAP_KEYNAME_MAXLEN) {
-		qb_log(LOG_ERR, "temp_key: failed, res: %d, nodeid: %u", res, nodeid);
+		qb_log(LOG_ERR, "temp_key: failed, res: %d, nodeid: " CS_PRI_NODE_ID, res, nodeid);
 		return ;
 	}
 	no_retries = 0;
@@ -278,11 +280,14 @@ static void _cs_cmap_members_key_changed (
 		return ;
 	}
 	*close_bracket = '\0';
-	res = _cs_ip_to_hostname(open_bracket, nodename);
-	if (res) {
+	if(conf[CS_NTF_NODNS]) {
 		strncpy(nodename, open_bracket, CS_MAX_NAME_LENGTH-1);
+	} else {
+		res = _cs_ip_to_hostname(open_bracket, nodename);
+		if (res) {
+			strncpy(nodename, open_bracket, CS_MAX_NAME_LENGTH-1);
+		}
 	}
-
 	_cs_node_membership_event(nodename, nodeid, (char *)new_value.data, open_bracket);
 	free(ip_str);
 }
@@ -761,7 +766,7 @@ _cs_dbus_init(void)
 #endif /* HAVE_DBUS */
 
 #ifdef ENABLE_SNMP
-static netsnmp_session *snmp_init (const char *target)
+static netsnmp_session *_cs_snmp_session_init (const char *target)
 {
 	static netsnmp_session *session = NULL;
 #ifndef NETSNMPV54
@@ -801,7 +806,7 @@ static netsnmp_session *snmp_init (const char *target)
 	return (session);
 }
 
-static inline void add_field (
+static void _cs_snmp_add_field (
 	netsnmp_pdu *trap_pdu,
 	u_char asn_type,
 	const char *prefix,
@@ -815,38 +820,63 @@ static inline void add_field (
 	}
 }
 
-static void
-_cs_snmp_node_membership_event(char *nodename, uint32_t nodeid, char *state, char* ip)
+static netsnmp_pdu *_cs_snmp_trap_pdu_init (const char *trap_oid)
 {
-	int ret;
-	char csysuptime[CS_TIMESTAMP_STR_LEN];
 	static oid snmptrap_oid[]  = { 1,3,6,1,6,3,1,1,4,1,0 };
 	static oid sysuptime_oid[] = { 1,3,6,1,2,1,1,3,0 };
-	time_t now = time (NULL);
-
+	char csysuptime[CS_TIMESTAMP_STR_LEN];
+	time_t now;
+	struct tm *now_tm;
 	netsnmp_pdu *trap_pdu;
-	netsnmp_session *session = snmp_init (snmp_manager);
-	if (session == NULL) {
-		qb_log(LOG_NOTICE, "Failed to init SNMP session.");
-		return ;
+
+	now = time (NULL);
+	if (now == ((time_t)-1)) {
+		qb_log(LOG_NOTICE, "Failed to get timestamp.");
+		return (NULL);
+	}
+
+	/* format uptime */
+	now_tm = localtime(&now);
+	if (now_tm == NULL || strftime (csysuptime, sizeof(csysuptime), "%s", now_tm) == 0) {
+		qb_log(LOG_NOTICE, "Failed to format timestamp.");
+		return (NULL);
 	}
 
 	trap_pdu = snmp_pdu_create (SNMP_MSG_TRAP2);
 	if (!trap_pdu) {
 		qb_log(LOG_NOTICE, "Failed to create SNMP notification.");
-		return ;
+		return (NULL);
 	}
 
 	/* send uptime */
-	snprintf (csysuptime, CS_TIMESTAMP_STR_LEN, "%ld", now);
 	snmp_add_var (trap_pdu, sysuptime_oid, sizeof (sysuptime_oid) / sizeof (oid), 't', csysuptime);
-	snmp_add_var (trap_pdu, snmptrap_oid, sizeof (snmptrap_oid) / sizeof (oid), 'o', SNMP_OID_TRAPS_NODE);
+	snmp_add_var (trap_pdu, snmptrap_oid, sizeof (snmptrap_oid) / sizeof (oid), 'o', trap_oid);
+
+	return (trap_pdu);
+}
+
+static void
+_cs_snmp_node_membership_event(char *nodename, uint32_t nodeid, char *state, char* ip)
+{
+	int ret;
+	netsnmp_pdu *trap_pdu;
+	netsnmp_session *session = _cs_snmp_session_init (snmp_manager);
+
+	if (session == NULL) {
+		qb_log(LOG_NOTICE, "Failed to init SNMP session.");
+		return ;
+	}
+
+	trap_pdu = _cs_snmp_trap_pdu_init(SNMP_OID_TRAPS_NODE);
+	if (trap_pdu == NULL) {
+		return ;
+	}
 
 	/* Add extries to the trap */
-	add_field (trap_pdu, ASN_OCTET_STR, SNMP_OID_OBJECT_NODE_NAME, (void*)nodename, strlen (nodename));
-	add_field (trap_pdu, ASN_UNSIGNED, SNMP_OID_OBJECT_NODE_ID, (void*)&nodeid, sizeof (nodeid));
-	add_field (trap_pdu, ASN_OCTET_STR, SNMP_OID_OBJECT_NODE_ADDR, (void*)ip, strlen (ip));
-	add_field (trap_pdu, ASN_OCTET_STR, SNMP_OID_OBJECT_NODE_STATUS, (void*)state, strlen (state));
+	_cs_snmp_add_field (trap_pdu, ASN_OCTET_STR, SNMP_OID_OBJECT_NODE_NAME, (void*)nodename, strlen (nodename));
+	_cs_snmp_add_field (trap_pdu, ASN_UNSIGNED, SNMP_OID_OBJECT_NODE_ID, (void*)&nodeid, sizeof (nodeid));
+	_cs_snmp_add_field (trap_pdu, ASN_OCTET_STR, SNMP_OID_OBJECT_NODE_ADDR, (void*)ip, strlen (ip));
+	_cs_snmp_add_field (trap_pdu, ASN_OCTET_STR, SNMP_OID_OBJECT_NODE_STATUS, (void*)state, strlen (state));
 
 	/* Send and cleanup */
 	ret = snmp_send (session, trap_pdu);
@@ -862,33 +892,23 @@ _cs_snmp_node_quorum_event(char *nodename, uint32_t nodeid,
 			   const char *state)
 {
 	int ret;
-	char csysuptime[20];
-	static oid snmptrap_oid[]  = { 1,3,6,1,6,3,1,1,4,1,0 };
-	static oid sysuptime_oid[] = { 1,3,6,1,2,1,1,3,0 };
-	time_t now = time (NULL);
-
 	netsnmp_pdu *trap_pdu;
-	netsnmp_session *session = snmp_init (snmp_manager);
+	netsnmp_session *session = _cs_snmp_session_init (snmp_manager);
+
 	if (session == NULL) {
 		qb_log(LOG_NOTICE, "Failed to init SNMP session.");
 		return ;
 	}
 
-	trap_pdu = snmp_pdu_create (SNMP_MSG_TRAP2);
-	if (!trap_pdu) {
-		qb_log(LOG_NOTICE, "Failed to create SNMP notification.");
+	trap_pdu = _cs_snmp_trap_pdu_init(SNMP_OID_TRAPS_QUORUM);
+	if (trap_pdu == NULL) {
 		return ;
 	}
 
-	/* send uptime */
-	sprintf (csysuptime, "%ld", now);
-	snmp_add_var (trap_pdu, sysuptime_oid, sizeof (sysuptime_oid) / sizeof (oid), 't', csysuptime);
-	snmp_add_var (trap_pdu, snmptrap_oid, sizeof (snmptrap_oid) / sizeof (oid), 'o', SNMP_OID_TRAPS_QUORUM);
-
 	/* Add extries to the trap */
-	add_field (trap_pdu, ASN_OCTET_STR, SNMP_OID_OBJECT_NODE_NAME, (void*)nodename, strlen (nodename));
-	add_field (trap_pdu, ASN_UNSIGNED, SNMP_OID_OBJECT_NODE_ID, (void*)&nodeid, sizeof (nodeid));
-	add_field (trap_pdu, ASN_OCTET_STR, SNMP_OID_OBJECT_QUORUM, (void*)state, strlen (state));
+	_cs_snmp_add_field (trap_pdu, ASN_OCTET_STR, SNMP_OID_OBJECT_NODE_NAME, (void*)nodename, strlen (nodename));
+	_cs_snmp_add_field (trap_pdu, ASN_UNSIGNED, SNMP_OID_OBJECT_NODE_ID, (void*)&nodeid, sizeof (nodeid));
+	_cs_snmp_add_field (trap_pdu, ASN_OCTET_STR, SNMP_OID_OBJECT_QUORUM, (void*)state, strlen (state));
 
 	/* Send and cleanup */
 	ret = snmp_send (session, trap_pdu);
@@ -904,35 +924,25 @@ _cs_snmp_link_faulty_event(char *nodename, uint32_t local_nodeid, uint32_t nodei
 		uint32_t iface_no, const char *state)
 {
 	int ret;
-	char csysuptime[20];
-	static oid snmptrap_oid[]  = { 1,3,6,1,6,3,1,1,4,1,0 };
-	static oid sysuptime_oid[] = { 1,3,6,1,2,1,1,3,0 };
-	time_t now = time (NULL);
-
 	netsnmp_pdu *trap_pdu;
-	netsnmp_session *session = snmp_init (snmp_manager);
+	netsnmp_session *session = _cs_snmp_session_init (snmp_manager);
+
 	if (session == NULL) {
 		qb_log(LOG_NOTICE, "Failed to init SNMP session.");
 		return ;
 	}
 
-	trap_pdu = snmp_pdu_create (SNMP_MSG_TRAP2);
-	if (!trap_pdu) {
-		qb_log(LOG_NOTICE, "Failed to create SNMP notification.");
+	trap_pdu = _cs_snmp_trap_pdu_init(SNMP_OID_TRAPS_LINK);
+	if (trap_pdu == NULL) {
 		return ;
 	}
 
-	/* send uptime */
-	sprintf (csysuptime, "%ld", now);
-	snmp_add_var (trap_pdu, sysuptime_oid, sizeof (sysuptime_oid) / sizeof (oid), 't', csysuptime);
-	snmp_add_var (trap_pdu, snmptrap_oid, sizeof (snmptrap_oid) / sizeof (oid), 'o', SNMP_OID_TRAPS_LINK);
-
 	/* Add extries to the trap */
-	add_field (trap_pdu, ASN_OCTET_STR, SNMP_OID_OBJECT_NODE_NAME, (void*)nodename, strlen (nodename));
-	add_field (trap_pdu, ASN_UNSIGNED, SNMP_OID_OBJECT_LOCAL_NODE_ID, (void*)&local_nodeid, sizeof (local_nodeid));
-	add_field (trap_pdu, ASN_UNSIGNED, SNMP_OID_OBJECT_NODE_ID, (void*)&nodeid, sizeof (nodeid));
-	add_field (trap_pdu, ASN_INTEGER, SNMP_OID_OBJECT_LINK_IFACE_NO, (void*)&iface_no, sizeof (iface_no));
-	add_field (trap_pdu, ASN_OCTET_STR, SNMP_OID_OBJECT_LINK_STATUS, (void*)state, strlen (state));
+	_cs_snmp_add_field (trap_pdu, ASN_OCTET_STR, SNMP_OID_OBJECT_NODE_NAME, (void*)nodename, strlen (nodename));
+	_cs_snmp_add_field (trap_pdu, ASN_UNSIGNED, SNMP_OID_OBJECT_LOCAL_NODE_ID, (void*)&local_nodeid, sizeof (local_nodeid));
+	_cs_snmp_add_field (trap_pdu, ASN_UNSIGNED, SNMP_OID_OBJECT_NODE_ID, (void*)&nodeid, sizeof (nodeid));
+	_cs_snmp_add_field (trap_pdu, ASN_INTEGER, SNMP_OID_OBJECT_LINK_IFACE_NO, (void*)&iface_no, sizeof (iface_no));
+	_cs_snmp_add_field (trap_pdu, ASN_OCTET_STR, SNMP_OID_OBJECT_LINK_STATUS, (void*)state, strlen (state));
 
 	/* Send and cleanup */
 	ret = snmp_send (session, trap_pdu);
@@ -965,16 +975,16 @@ _cs_snmp_init(void)
 static void
 _cs_syslog_node_membership_event(char *nodename, uint32_t nodeid, char *state, char* ip)
 {
-	qb_log(LOG_NOTICE, "%s[%d] ip:%s %s", nodename, nodeid, ip, state);
+	qb_log(LOG_NOTICE, "%s[" CS_PRI_NODE_ID "] ip:%s %s", nodename, nodeid, ip, state);
 }
 
 static void
 _cs_syslog_node_quorum_event(char *nodename, uint32_t nodeid, const char *state)
 {
 	if (strcmp(state, "quorate") == 0) {
-		qb_log(LOG_NOTICE, "%s[%d] is now %s", nodename, nodeid, state);
+		qb_log(LOG_NOTICE, "%s[" CS_PRI_NODE_ID "] is now %s", nodename, nodeid, state);
 	} else {
-		qb_log(LOG_NOTICE, "%s[%d] has lost quorum", nodename, nodeid);
+		qb_log(LOG_NOTICE, "%s[" CS_PRI_NODE_ID "] has lost quorum", nodename, nodeid);
 	}
 }
 
@@ -982,16 +992,16 @@ static void
 _cs_syslog_application_connection_event(char *nodename, uint32_t nodeid, char* app_name, const char *state)
 {
 	if (strcmp(state, "connected") == 0) {
-		qb_log(LOG_NOTICE, "%s[%d] %s is now %s to corosync", nodename, nodeid, app_name, state);
+		qb_log(LOG_NOTICE, "%s[" CS_PRI_NODE_ID "] %s is now %s to corosync", nodename, nodeid, app_name, state);
 	} else {
-		qb_log(LOG_NOTICE, "%s[%d] %s is now %s from corosync", nodename, nodeid, app_name, state);
+		qb_log(LOG_NOTICE, "%s[" CS_PRI_NODE_ID "] %s is now %s from corosync", nodename, nodeid, app_name, state);
 	}
 }
 
 static void
 _cs_syslog_link_faulty_event(char *nodename, uint32_t our_nodeid, uint32_t nodeid, uint32_t iface_no, const char *state)
 {
-	qb_log(LOG_NOTICE, "%s[%d] link %u to node %u is now %s", nodename, our_nodeid, iface_no, nodeid, state);
+	qb_log(LOG_NOTICE, "%s[" CS_PRI_NODE_ID "] link %u to node " CS_PRI_NODE_ID " is now %s", nodename, our_nodeid, iface_no, nodeid, state);
 }
 
 static void
@@ -1258,6 +1268,7 @@ _cs_usage(void)
 		"        -o     : Print events to stdout (turns on -l).\n"\
 		"        -s     : Send SNMP traps on all events.\n"\
 		"        -m     : Set the SNMP Manager IP address (defaults to localhost).\n"\
+		"        -n     : No reverse DNS lookup on cmap member change events.\n"\
 		"        -d     : Send DBUS signals on all events.\n"\
 		"        -h     : Print this help.\n\n");
 }
@@ -1273,7 +1284,7 @@ main(int argc, char *argv[])
 	conf[CS_NTF_SNMP] = QB_FALSE;
 	conf[CS_NTF_DBUS] = QB_FALSE;
 
-	while ((ch = getopt (argc, argv, "c:floshdm:")) != EOF) {
+	while ((ch = getopt (argc, argv, "c:floshdnm:")) != EOF) {
 		switch (ch) {
 			case 'c':
 				strncpy(snmp_community_buf, optarg, sizeof (snmp_community_buf));
@@ -1292,6 +1303,9 @@ main(int argc, char *argv[])
 				snmp_manager_buf[sizeof (snmp_manager_buf) - 1] = '\0';
 				snmp_manager = snmp_manager_buf;
 				break;
+			case 'n':
+				conf[CS_NTF_NODNS] = QB_TRUE;
+				break;
 			case 'o':
 				conf[CS_NTF_LOG] = QB_TRUE;
 				conf[CS_NTF_STDOUT] = QB_TRUE;
